/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file
 * Defines a copy+fill algorithm
 */

///////////////////////////////////////////////////////////////////////////////
// include guard
///////////////////////////////////////////////////////////////////////////////

#pragma once



///////////////////////////////////////////////////////////////////////////////
// function definition
///////////////////////////////////////////////////////////////////////////////

/// Copies a sequence to a destination buffer, then fills any remaining space
/// in the destination w/ a fill value.
template<class ToIter, class FromIter, class Fill>
void CopyFill(
	ToIter to,        //<Start copying into here
	ToIter to_end,    //<Stop copying when this is reached
	FromIter from,    //<Start copying from here
	FromIter from_end,//<Stop copying when this is reached
	Fill fill )       //<Fill value
{
	//copy until run out of input or run out of room
	while( to != to_end && from != from_end )
	{
		*to++ = *from++;
	}

	//if there's still room left fill it
	while( to != to_end )
	{
		*to++ = fill;
	}
}


