/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file
 * CppUnit main
 */

#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/ui/text/TextTestRunner.h"
#include "cppunit/TestResult.h"
#include "cppunit/Protector.h"
#include "ApexException.h"
#include "StringUtils.h"
#include "cppunit/Exception.h"
#include "cppunit/extensions/TypeInfoHelper.h"
#include <typeinfo>
#include <sstream>


extern void UtilsRegisterUnitTests();


namespace
{
	enum { add_file_name = false };


	template<class T>
	std::string short_descr( const T& e, const char* base_name )
	{
		return std::string( "uncaught exception of type " ) +
#if CPPUNIT_USE_TYPEINFO_NAME
			CppUnit::TypeInfoHelper::getClassName( typeid( e ));
#else
			base_name + " (or derived)";
#endif
	}
}



struct TklProtector : public CppUnit::Protector
{
	bool protect(
		const CppUnit::Functor& functor,
		const CppUnit::ProtectorContext& context )
	{
		try
		{
			return functor();
		}
		catch( const CppUnit::Exception& failure )
		{
			reportFailure( context, failure );
		}
		catch( const CApexException& e )
		{
			CppUnit::Message message( short_descr( e, "CApexException" ));
			message.addDetail( CStringUtils::Narrow( e.GetDescription( add_file_name )));
			std::ostringstream o;
			o << "Thrown from " << CStringUtils::Narrow( e.GetFileName()) << ", line " << e.GetLineNumber();
			message.addDetail( o.str());
			reportError( context, message );
		}
		catch( const std::exception& e )
		{
			CppUnit::Message message( short_descr( e, "std::exception" ), e.what());
			reportError( context, message );
		}
		catch( ... )
		{
			reportError( context, CppUnit::Message( "uncaught exception of unknown type" ));
		}

		return false;
	}
};



int main( int, char ** )
{
	enum {	pause_at_end = true,
			print_result = true,
			print_progress = true };

	UtilsRegisterUnitTests();

	CppUnit::TextTestRunner runner;
	runner.addTest( CppUnit::TestFactoryRegistry::getRegistry().makeTest());
	runner.eventManager().pushProtector( new TklProtector );

	return runner.run( "", pause_at_end, print_result, print_progress );
}

