#include "uphil.h"
#include <libusb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <err.h>

/* before firmware's loaded */
#define UNLOADED_VENDOR_ID (0x08a9) // USBee
#define UNLOADED_PRODUCT_ID (0x0302) // Uphil


typedef struct failures_t failures_t;

struct failures_t
{
    int r;
};

static int _load_visited(void *p, libusb_device *device)
{
    failures_t *failures = (failures_t *)p;
    int r;
    int b = libusb_get_bus_number(device);
    int a = libusb_get_device_address(device);
    warnx("Loading firmware into /dev/bus/usb/%03d/%03d", b, a);
    r = uphil_load_firmware2(device);
    warnx("%s", (r ? "FAILED" : "Done"));
    failures->r |= r;
    return 0; // keep iterating
}

static int _load_all(void)
{
    failures_t failures = { 0 };
    int l, r = -1;

    if ((l = libusb_init(NULL))) {
        warnx("%s", libusb_strerror(l));
        goto exit;
    }

    uphil_visit_unloaded(NULL, _load_visited, &failures);
    r = failures.r;

exit:
    libusb_exit(NULL);
    return r;
}

static int _load_one(int bus, int address)
{
    int r;

    warnx("Loading firmware into /dev/bus/usb/%03d/%03d", bus, address);
    r = uphil_load_firmware(bus, address);
    warnx("%s", (r ? "FAILED" : "Done"));
    return r;
}

static void _usage()
{
    printf("load /dev/bus/usb/<bus>/<device>, or\n"
        "load all\n");
}

int main(int argc, char *argv[])
{
    int bus, address;

    if (argc < 2) {
        _usage();
        return 0;
    }

    if (0 == strcmp("all", argv[1])) {
        return _load_all();
    }

    if (2 == sscanf(argv[1], "/dev/bus/usb/%d/%d", &bus, &address)) {
        return _load_one(bus, address);
    }

    _usage();
    return -1;
}
